package net.cattaka.rdbassistant.gui.config;

import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JFileChooser;
import javax.swing.JPanel;

import net.cattaka.rdbassistant.RdbaConfigConstants;
import net.cattaka.rdbassistant.config.RdbaConfig;
import net.cattaka.swing.datainputpanel.DIPInfo;
import net.cattaka.swing.datainputpanel.DIPInfoFile;
import net.cattaka.swing.datainputpanel.DIPInfoFont;
import net.cattaka.swing.datainputpanel.DIPInfoSelect;
import net.cattaka.swing.datainputpanel.DataInputPanel;
import net.cattaka.swing.util.LookAndFeelBundle;
import net.cattaka.util.ExceptionHandler;
import net.cattaka.util.MessageBundle;

public class RdbaConfigEditorPanel extends JPanel {
	private static final long serialVersionUID = 1L;
	private DataInputPanel dataInputPanel;
	
	private DIPInfoFont fontForEditor;
	private DIPInfoFont fontForTable;
	private DIPInfoFile sqlQuickAccess;
	private DIPInfoFile scriptWorkDir;
	private DIPInfoFile toolsJar;
	private DIPInfoFile jdbcJarMysql;
	private DIPInfoFile jdbcJarOracle;
	private DIPInfoSelect lookAndFeel;
	private DIPInfoSelect logLevel;
	
	class ActionListenerEx implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			if (e.getActionCommand().equals("approve")) {
				if (dataInputPanel.isValidData()) {
					doApprove();
				} else {
					dataInputPanel.validateData();
				}
			} else if (e.getActionCommand().equals("cancel")) {
				doCancel();
			}
		}
	}
	
	public RdbaConfigEditorPanel() {
		makeLayout();
	}
	
	private void makeLayout() {
		fontForEditor = new DIPInfoFont(MessageBundle.getMessage("font_for_editor"), new Font(RdbaConfigConstants.DEFAULT_FONT_EDITOR, 0 , 12));
		fontForTable = new DIPInfoFont(MessageBundle.getMessage("font_for_table"), new Font(RdbaConfigConstants.DEFAULT_FONT_EDITOR, 0 , 12));
		sqlQuickAccess = new DIPInfoFile(MessageBundle.getMessage("sql_quick_access"), "", DIPInfoFile.MODE_OPEN);
		scriptWorkDir = new DIPInfoFile(MessageBundle.getMessage("script_work_dir"), "", DIPInfoFile.MODE_OPEN);
		toolsJar = new DIPInfoFile(MessageBundle.getMessage("tools_jar"), "", DIPInfoFile.MODE_OPEN);
		jdbcJarMysql = new DIPInfoFile(MessageBundle.getMessage("jdbc_jar_mysql"), "", DIPInfoFile.MODE_OPEN);
		jdbcJarOracle = new DIPInfoFile(MessageBundle.getMessage("jdbc_jar_oracle"), "", DIPInfoFile.MODE_OPEN);
		lookAndFeel  = new DIPInfoSelect(MessageBundle.getMessage("look_and_feel"), LookAndFeelBundle.getLookAndFeelNames(), LookAndFeelBundle.getLookAndFeelClassNames(), LookAndFeelBundle.getDefaultLookAndFeelName());
		logLevel = new DIPInfoSelect(MessageBundle.getMessage("log_level"), ExceptionHandler.getPriorityNames(), ExceptionHandler.Priority.WARNING.name());
		
		sqlQuickAccess.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		scriptWorkDir.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		
		DIPInfo[] dipis = new DIPInfo[] {
				fontForEditor,
				fontForTable,
				sqlQuickAccess,
				scriptWorkDir,
				toolsJar,
				jdbcJarMysql,
				jdbcJarOracle,
				lookAndFeel,
				logLevel
		};
		
		ActionListener al = new ActionListenerEx();
		dataInputPanel = new DataInputPanel(dipis);
		dataInputPanel.getButtonApprove().addActionListener(al);
		dataInputPanel.getButtonApprove().setActionCommand("approve");
		dataInputPanel.getButtonCancel().addActionListener(al);
		dataInputPanel.getButtonCancel().setActionCommand("cancel");

		this.setLayout(new GridLayout());
		this.add(dataInputPanel);
	}
	
	public void loadRdbaConfig(RdbaConfig rdbaConfig) {
		fontForEditor.setValue(rdbaConfig.getFontForEditor());
		fontForTable.setValue(rdbaConfig.getFontForTable());
		sqlQuickAccess.setValue(rdbaConfig.getSqlQuickAccessRoot());
		scriptWorkDir.setValue(rdbaConfig.getScriptRoot());
		toolsJar.setStringValue(rdbaConfig.getToolsJar());
		jdbcJarMysql.setStringValue(rdbaConfig.getJdbcJarMysql());
		jdbcJarOracle.setStringValue(rdbaConfig.getJdbcJarOracle());
		lookAndFeel.setObjectValue(rdbaConfig.getLookAndFeel());
		logLevel.setStringValue(rdbaConfig.getLogLevel());
	}

	public void saveRdbaConfig(RdbaConfig rdbaConfig) {
		rdbaConfig.setFontForEditor(fontForEditor.getFontValue());
		rdbaConfig.setFontForTable(fontForTable.getFontValue());
		rdbaConfig.setSqlQuickAccessRoot(sqlQuickAccess.getFileValue());
		rdbaConfig.setScriptRoot(scriptWorkDir.getFileValue());
		rdbaConfig.setToolsJar(toolsJar.getStringValue());
		rdbaConfig.setJdbcJarMysql(jdbcJarMysql.getStringValue());
		rdbaConfig.setJdbcJarOracle(jdbcJarOracle.getStringValue());
		rdbaConfig.setLookAndFeel(lookAndFeel.getObjectValue().toString());
		rdbaConfig.setLogLevel(logLevel.getStringValue());
	}

	public void doApprove() {
		
	}
	public void doCancel() {
		
	}
}
