package net.cattaka.rdbassistant.gui.sql;

import java.awt.Component;

import javax.swing.JTabbedPane;

import net.cattaka.rdbassistant.bean.RdbaSingletonBundle;
import net.cattaka.rdbassistant.config.RdbaConfig;
import net.cattaka.rdbassistant.core.RdbaConnection;
import net.cattaka.rdbassistant.gui.RdbaGuiInterface;
import net.cattaka.rdbassistant.gui.RdbaMessage;
import net.cattaka.util.MessageBundle;

public class RdbaSqlAssistTabbedPanel extends JTabbedPane implements RdbaGuiInterface {
	private static final long serialVersionUID = 1L;
	private RdbaGuiInterface parentComponent;
	private RdbaSqlSelectPanel rdbaSelectPanel;
	private RdbaSqlQuickAccessPanel rdbaSqlQuickAccessPanel;
	
	public RdbaSqlAssistTabbedPanel(RdbaGuiInterface parentComponent) {
		this.parentComponent = parentComponent;
		makeLayout();
	}
	
	private void makeLayout() {
		this.rdbaSelectPanel = new RdbaSqlSelectPanel(this);
		this.rdbaSqlQuickAccessPanel = new RdbaSqlQuickAccessPanel(this);
		
		this.addTab(MessageBundle.getMessage("db_info"), this.rdbaSelectPanel);
		this.addTab(MessageBundle.getMessage("quick_access"), this.rdbaSqlQuickAccessPanel);
	}

	public void setRdbaConnection(RdbaConnection rdbaConnection) {
		rdbaSelectPanel.setRdbaConnection(rdbaConnection);
	}

	/** {@link RdbaGuiInterface} */
	public void doGuiLayout() {
		this.rdbaSelectPanel.doGuiLayout();
	}

	/** {@link RdbaGuiInterface} */
	public RdbaConfig getRdbaConfig() {
		return parentComponent.getRdbaConfig();
	}

	/** {@link RdbaGuiInterface} */
	public RdbaSingletonBundle getRdbaSingletonBundle() {
		return parentComponent.getRdbaSingletonBundle();
	}

	/** {@link RdbaGuiInterface} */
	public void relayRdbaMessage(RdbaMessage rdbaMessage) {
		Component comp = this.getSelectedComponent();
		if (comp != null && comp instanceof RdbaGuiInterface) {
			((RdbaGuiInterface)comp).relayRdbaMessage(rdbaMessage);
		}
	}

	/** {@link RdbaGuiInterface} */
	public void reloadRdbaConfig() {
		this.rdbaSelectPanel.reloadRdbaConfig();
		this.rdbaSqlQuickAccessPanel.reloadRdbaConfig();
	}

	/** {@link RdbaGuiInterface} */
	public void sendRdbaMessage(RdbaMessage rdbaMessage) {
		parentComponent.sendRdbaMessage(rdbaMessage);
	}

	/** {@link RdbaSqlSelectPanel} {@link RdbaSqlQuickAccessPanel} */
	public void setRdbaSqlEditorTabbedPanel(RdbaSqlEditorTabbedPanel rdbaSqlEditorPanel) {
		rdbaSelectPanel.setRdbaSqlEditorTabbedPanel(rdbaSqlEditorPanel);
		rdbaSqlQuickAccessPanel.setRdbaSqlEditorTabbedPanel(rdbaSqlEditorPanel);
	}
}
